# -*- coding: utf-8 -*- 
##############################################################################
#
# Copyright (c) Camptocamp SA - Joel Grand-Guillaume
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
from mx import DateTime
import netsvc
from osv import fields
from osv import osv


#  To avoid dependances on c2c_timesheet_report, put the ended date on hr_employee
class Employee (osv.osv):
    """Override emplyee class and add ended field"""
    _inherit = "hr.employee"
    _columns  = {
                    'ended' : fields.date('Ended on'),
                    'regime' : fields.float('regime', size=256)
                }
        
Employee()


class report_account_analytic_planning_line(osv.osv):
    """Represents planning line and 
      Add departement field (function) on lines"""
    _inherit = "report_account_analytic.planning.line"
        
    def _check_activity_and_analytic(self,cr,uid,vals,ids={}):
        """Validate a planning line activie and account, we ensure that
        the link between these to is correct also
        validate the date of the line"""
        activities_obj=self.pool.get('project.activity_al')
        
        # Both activity and account are changed or create
        if vals.has_key('account_id') and vals.has_key('activity_id'):
            #take the new account wich have activity_ids define for the line
            concerned_account_obj=self.pool.get('account.analytic.account')\
                .browse(cr,uid,vals['account_id'])
            #take the new activity 
            concerned_activity_id=vals['activity_id']
        # Only account_id is changed
        elif vals.has_key('account_id') \
            and not vals.has_key('activity_id'):
            #take the new account wich have activity_ids define for the line
            concerned_account_obj=self.pool.get('account.analytic.account')\
                .browse(cr,uid,vals['account_id'])
            #take the activity saved in DB
            # take the recorded planning line
            pl_line=self.browse(cr,uid,ids)[0]
            concerned_activity_id=pl_line.activity_id.id
        # Only activity_id is changed
        elif vals.has_key('activity_id') and not vals.has_key('account_id'):
            # take the recorded planning line
            pl_line=self.browse(cr,uid,ids)[0]
            concerned_account_obj=pl_line.account_id
            #take the new activity 
            concerned_activity_id=vals['activity_id']
        # nothing is changed in activities and account
        else:
            return True
        acc_who_matters=activities_obj._get_first_AA_wich_have_activity(
                                                            cr,
                                                            uid,
                                                            concerned_account_obj
                                                        )
        # Take all activities
        allowed_activity_for_proj=[]
        if acc_who_matters:
            for i in acc_who_matters.activity_ids:
                allowed_activity_for_proj.append(i.id)
        # If the chossen activity_id for the line is not in the activity 
        #define in account or his parent 
        # Raise error...
        if not concerned_activity_id in allowed_activity_for_proj:
            raise osv.except_osv(
                                    'Error !', 
                                    'For each line you can only set an activity wich \
                                    is defined for this account (or in his parent) ! \
                                    Change the activity or the account and try again.')
        else:
            return True
     
    # On create check the activity !
    def create(self, cr, uid, vals, context={}):
        if not context:
            context={}

        self._check_activity_and_analytic(cr,uid,vals)
        
        if vals['date_from']>vals['date_to']:
            raise osv.except_osv('Error !', 'The start date of a line must be earlier or equal to the end date !')

        return super(report_account_analytic_planning_line,self).create(cr, uid, vals)
    
    
    def straight_unlink(self, cr, uid, ids, context=None):
        """ delete a line without any check 
        (used by hr_holidays.py to handle lines 
        associated with holidays request)"""
        return super(report_account_analytic_planning_line,self)\
            .unlink(cr, uid, ids, context=context)
    
    def unlink(self, cr, uid, ids, context=None):    
        """prohibit to deletion of a planning 
            line generated by holiday requests """
        
        #if a deleted line belongs to a holiday request, we refuse to delete
        lines = self.browse(cr, uid, ids, context)
        for l in lines:
            if l.holiday_request and l.planning_id.default_planning:
                raise osv.except_osv(
                            'Lines not deleted', 
                            """You can not delete a planning line generated 
                            by a holidays request.
                            To edit it, edit the corresponding holidays request"""
                        );
        return super(report_account_analytic_planning_line,self)\
            .unlink(cr, uid, ids, context=context)
        
    
    def write(self, cr, uid, ids, vals, context=None):
        """ check the activity match the analytic account 
            prohibit to edition of a planning line generated 
            by holiday requests. Does not allows to write 
            line genereated by holyday request
        """
        
        
        if not context:
            context={}
        self._check_activity_and_analytic(cr,uid,vals,ids)
        
        #if a edit line belongs to a holiday request, we refuse to save
        lines = self.browse(cr, uid, ids, context)
        for l in lines:
            if l.holiday_request and l.planning_id.default_planning:
                raise osv.except_osv(
                                'Lines not saved', 
                                """You can not edit a planning line generated 
                                 by a holidays request. To change it, edit the 
                                 corresponding holidays request"""
                                    );
                
        
                        
        # Check date_from<= date_to
        if vals.has_key('date_from') and vals.has_key('date_to') \
            and vals['date_from']>vals['date_to']:
            raise osv.except_osv(
                                    'Error !', 
                                    """The start date of a line must be 
                                    earlier or equal to the end date !"""
                                )
            
        return super(report_account_analytic_planning_line,self).write(
                                                                        cr, 
                                                                        uid, 
                                                                        ids, 
                                                                        vals, 
                                                                        context=context
                                                                    )
    
    
    def _calc_unified_amount(self, cr, uid, ids, name, arg, context={}):
        res = {}
        for pl_line in self.browse(cr, uid, ids):
            factor=pl_line.amount_unit.factor
            amount=pl_line.amount
            res[pl_line.id] = factor * amount
        for id in ids:
            res[id] = round(res.get(id, 0.0),2)
        return res

 
    _columns = {
        'department_id': fields.many2one('hr.department', 'Department', select=True),
        'product_id': fields.many2one('product.product', 'Job / Product', required=True),
        'activity_id': fields.many2one('project.activity_al', 'Activity', required=True),
        'date_from':fields.date('Start Date', required=True),
        'date_to':fields.date('End date',required=True),
        # 'txt_date_from' : fields.char("Start date", size=10, required=True),
        # 'txt_date_to' : fields.char("End date", size=10, required=True),
        'name' : fields.char("Name", size=64), #required=True),
        'unified_amount': fields.function(_calc_unified_amount, store=True,method=True, type='float', string='Unified Quantity',help='Convert the amount with the factor of the unit choosen for the line (Ex: 1 day : factor 8.4, 1 hour : factor 1 => if there is 1 day and 2 hours, the total will be 8.4*1 + 2*1 = 10.4. Here it will be in hours.)'),
        'manager': fields.many2one('res.users', 'Assigned by', required=True),
        'holiday_request': fields.many2one('hr.holidays', 'Generated by Holidays Request'),
           }
    _defaults = {
        # 'date_from' : lambda *a: (DateTime.now() + DateTime.RelativeDateTime(days=+6,weekday=(DateTime.Sunday,0))).strftime('%Y-%m-%d'),
        #    'date_to' : lambda *a: (DateTime.now() + DateTime.RelativeDateTime(days=+6,weekday=(DateTime.Sunday,0))).strftime('%Y-%m-%d'),
        'manager' : lambda obj, cr, uid, context: uid,
        'planning_id': lambda obj, cr, uid, context: obj.pool.get('report_account_analytic.planning')._get_default_planning(cr, uid, context),

    }
    def _get_product_from_employee(self, cr, uid, user_id, context={}):
        query = """SELECT p.id 
                    FROM product_product p 
                    left join hr_employee e on(p.id = e.product_id) 
                    WHERE e.user_id=%d LIMIT 1"""% user_id
        cr.execute(query)
        res = cr.dictfetchone()
        if res:
            return res['id']
        else:
            return False
        
    #  On user change => try to get the first departement found
    #  Put the right product for the current account
    def onchange_user_id(self, cr, uid, id, user_id, account_id, product_id,amount_unit,amount,context={}, *args):

        result = {'value':{}}
        if not (user_id):
            #avoid a useless call to super
            return result 

        # If account, find the right product for the user defined by analytic user function
        if account_id:
            # If product already there keep it
            if not product_id:
                #get the browse record related to user_id and account_id
                temp = self.pool.get('hr.analytic.timesheet')._get_related_user_account_recursiv(cr,uid,user_id,account_id)
                if temp:
                    #add the value from the new relation analytic_user_funct_grid
                    r = self.pool.get('analytic_user_funct_grid').browse(cr, uid, temp)[0]
                    result['value']['product_id'] = r.product_id.id
                # If not define, take the employee product
                else:
                    result['value']['product_id'] = self._get_product_from_employee(cr,uid,user_id,context)
            else:
                pass
        # If no account define, take employee product
        else:
            # If product already there keep it
            if not product_id:
                result['value']['product_id'] = self._get_product_from_employee(cr,uid,user_id,context)
            else:
                pass
        # Take the first department as a proposition
        query2 = """SELECT department_id 
            FROM hr_department_user_rel 
            WHERE user_id=%d LIMIT 1"""% user_id

        cr.execute(query2)
        res2 = cr.dictfetchone()
        
        if res2:
            result['value']['department_id']=res2['department_id']
            # return {'value':{'department_id': res2['id']}}
        else:
            result['value']['department_id']=False
        
        # Take right amount unit for product if there is one
        if result['value'].has_key('product_id'):
            uom = self.product_id_change(cr,uid,id,result['value']['product_id'],amount_unit,amount)
            if uom.has_key('value'):
                result['value']['amount_unit']=uom['value']['amount_unit']
            else:
                result['value']['amount_unit']=False
        return result
        
    # On change oproduct => set default uom
    def product_id_change(self, cr, uid, ids, product,amount_unit,amount,context={},*args):
        product_uom_obj = self.pool.get('product.uom')
        product_obj = self.pool.get('product.product')
        # Return nothing
        if not product:
            # return {'value': {'amount_unit':False}}
            return {}
        # Product choosen
        product = product_obj.browse(cr, uid, product, context=context)
            
        # If amount_unit and product.amount_unit is equal, do nothing, let the amount
        if amount_unit and amount_unit == product.uom_id.id:
            return {}
        # Convert amount with unit factor if unit differ from product
        elif amount_unit and amount and amount_unit<>product.uom_id.id:
            unit_obj_product=product_uom_obj.browse(cr,uid,product.uom_id.id)
            unit_obj_current=product_uom_obj.browse(cr,uid,amount_unit)   
            new_value=(unit_obj_current.factor*amount)/unit_obj_product.factor
            
            return {'value': {'amount_unit':product.uom_id.id,'amount':new_value}}

        # Change unit only because no value for amount         # not value, we enter product only
        elif (not amount and amount_unit and amount_unit<>product.uom_id.id) or (not amount_unit and not amount):
            uom = product.uom_id.id
            return {'value': {'amount_unit':uom}}

        else:
            return {}


    def onchange_date_from(self,cr,uid,ids,date_from,date_to,context={},*args):
        res={'value':{}}
        # Handle date_to prior to date_from on date_from change =>re-set the date_to X days after
        # where X is the difference between old and new value
        
        if date_to and date_from and date_from>date_to:
            res['value']['date_from']=date_from
            # res=False
        if not date_to and date_from:
            res['value']['date_to']=date_from
        return res
        
    def onchange_date_to(self,cr,uid,ids,date_from,date_to,context={},*args):
        res={'value':{}}
        # Handle date like 0208 => 02/08/2008 and 120808 => 12/08/2008
        # and 2.8  -> 02/08/2008 and 2.8.8 -> 02/08/2008
        # date_to=self._convert_simple_date(cr,uid,date_to)
        # Handle date_to prior to date_fromon date_to change => set it to date_from
        if date_to and date_from and date_from>date_to:
            res['value']['date_to']=False
            # res=False
        return res

    def onchange_activity(self,cr,uid,ids,activity_id,account_id,context={},*args):
        res={'value':{}}
        # Handle selection of activity not related to account_id or his parent 
        # (possible cause miss something to hadle this properly in context and name_search)
        
        
        return res
        
report_account_analytic_planning_line()

#######################################################################################
#  report by product from Tiny
#######################################################################################
class report_account_analytic_planning_stat_product(osv.osv):

    def _sum_amount_product(self, cr, uid, ids, name, args, context):
        result = {}
        for line in self.browse(cr, uid, ids, context):
            result[line.id] = 0.0
            if line.product_id:
                cr.execute('select sum(unit_amount) from account_analytic_line where product_id=%s and date>=%s and date<=%s', (str(line.product_id.id),line.planning_id.date_from,line.planning_id.date_to))
                result[line.id] = cr.fetchone()[0]
        return result

    def init(self, cr):
        cr.execute("""
            create or replace view report_account_analytic_planning_stat_product as (
                select
                    min(l.id) as id,
                    p.id as product_id,
                    sum(l.amount*u.factor) as quantity,
                    l.planning_id
                from
                    report_account_analytic_planning_line l
                left join
                    product_uom u on (l.amount_unit = u.id)
                left join
                    product_product p on (p.id=l.product_id)
                group by
                    planning_id, p.id
            )
        """)

    _name = "report_account_analytic.planning.stat.product"
    _description = "Planning product stat"
    _rec_name = 'product_id'
    _auto = False
    _log_access = False
    _columns = {
        'planning_id': fields.many2one('report_account_analytic.planning', 'Planning', required=True),
        'product_id': fields.many2one('product.product', 'Post / Product', required=True),
        'quantity': fields.float('Planned', required=True),
        'sum_amount_real': fields.function(_sum_amount_product, method=True, string='Timesheet'),
    }

report_account_analytic_planning_stat_product()

#######################################################################################
#  report by product from Tiny
#######################################################################################
class report_account_analytic_planning(osv.osv):
    _inherit = "report_account_analytic.planning"
    _columns = {
        'stat_product_ids': fields.one2many('report_account_analytic.planning.stat.product', 'planning_id', 'Planning by Post / Product', readonly=True),
        'default_planning': fields.boolean('Default planning for employee'),
        'default_uom'    : fields.many2one('product.uom', 'Default Uom', required=True),
       

    }
    _defaults = {
        'default_planning': lambda *a: 0,
    }
    # Return default planning id
    def _get_default_planning(self, cr, uid, context={}):
        res=self.search(cr,uid,[('default_planning','=',True),('state','=','open')])
        if res and isinstance(res,list):
            res=res[0]
        else:
            res=False
        return res
    
    # Return False if there is more than one open planning with default_planning checked
    def _check_only_one_default(self, cr, uid, ids):
        lines = self.search(cr, uid, [('default_planning','=',True),('state','=','open')])
        if len(lines)>1:
                return False
        return True

    _constraints = [
        (_check_only_one_default, 'Error ! You can not have two planning with the state open and the Default planning for employee checked.', ['default_planning'])
    ]
report_account_analytic_planning()

class task(osv.osv):
    _inherit = "project.task"
    def _compute_line_name(self, cr, uid, ids, name, arg, context={}):
        res = {}
        if not ids :
            return {}
        
        for task in self.browse(cr, uid, ids):
            name = task.planning_line_id.note
            if name :
                if len(name) > 30 :
                    name = name[0:29]
            else :
                name = 'no note in PL'
            print name
            res[task.id] = name
        return res
    _columns = {
            'planning_line_id' : fields.many2one('report_account_analytic.planning.line', 'planning'),
            'cp_name' : fields.function(_compute_line_name, method=True, type='char', size=256)
    }
task()
