# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA
# Author: Arnaud Wüst
#
#
#    This file is part of the c2c_planning_management module
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from c2c_reporting_tools.translation import _

from report.interface import report_int
from c2c_reporting_tools.flowables.timebox_chart import  *
import pooler
import planning_by_type
from datetime import datetime, timedelta
import time

class Planning_by_description(planning_by_type.Planning_by_type):
                
    report_name = "Planning by description"
    
    def build_chart_datas(self):
        """ retrieve and format the planning lines to a 
        structure that can be used to draw the chart """
            
        periods = {}
        
        planning_names = {}
        account_names = {}
#        function_names = {}
        
       
        #parse all lines and sort them in a tree:  planning _id-> description ->periods_list
        lines = pooler.get_pool(self.cr.dbname).get(
                                'report_account_analytic.planning.line'
                            ).browse(
                                        self.cr, 
                                        self.uid, 
                                        self.ids
                                    )
        for line in lines:        
            if line.note == False:
                line.note = line.activity_id.name    
            #does the planning dictionnary already exists in the tree.
            # If not, we must init it
            if line.planning_id.id not in periods:
                periods[line.planning_id.id] = {}

            #does the project dictionnary alrady exists in the tree.
            # If not, we must init it
            if line.account_id.id not in periods[line.planning_id.id]:
                periods[line.planning_id.id][line.account_id.id] = {}              
                
            #does the description dictionnary alrady exists in the tree. 
            #If not, we must init it
            if line.note not in periods[line.planning_id.id][line.account_id.id]:
                periods[line.planning_id.id][line.account_id.id][line.note] = []
                    
            #store the period of the line in the planning/tree
            uname = 'unasigned'
            if type(line.user_id) != osv.orm.browse_null:
                uname = line.user_id.name    
            periods[line.planning_id.id][line.account_id.id][line.note].\
                append(
                        (
                            time.strptime(line.date_from, "%Y-%m-%d"), 
                            time.strptime(line.date_to, "%Y-%m-%d"), 
                            str(int(line.unified_amount)), 
                            uname, 
                            line.activity_id.name
                        )
                    )
            
#            #also store the names of the elements
            planning_names[line.planning_id.id] = line.planning_id.name
            account_names[line.account_id.id] = line.account_id.name
        del lines
        
        
        charts_datas = {}

        date_start, date_end = self.get_form_boundary_dates()
        #now the line are sorted, we can build the data
        for planning_id in periods:
            #one chart by planning
            charts_datas[planning_id] = TimeBoxesChartLegendBuilder(
                                                    "", 
                                                    _("Users:"), 
                                                    date_start, 
                                                    date_end
                                                )            
            
            #descriptions are grouped by project
            for account_id in periods[planning_id]:
                charts_datas[planning_id].\
                    append_Y_item("<b>"+account_names[account_id]+"</b>")                
                        
                for line_note in periods[planning_id][account_id]:
                    #each phase is a title 
                    item_id = charts_datas[planning_id].append_Y_item(line_note, 1)                
                    for period in periods[planning_id][account_id][line_note]:
                        #each period is a box in the chart
                        period_date_start = period[0]
                        period_date_end = period[1]
                        period_time= period[2]
                        period_legend_text = period[3]
                        period_text = period[4]
                        charts_datas[planning_id].add_period(
                                            item_id, 
                                            period_date_start,
                                             period_date_end,
                                             None, 
                                             period_time+"h - "+period_text, 
                                             period_legend_text
                                        )
            
            #retrieve boundary dates to put them in the title
            (boundary_date_start, boundary_date_end) = charts_datas[planning_id].\
                get_boundary_dates()
            #add dates to the chart title
            title = planning_names[planning_id]+" from "+\
                boundary_date_start.strftime("%d.%m.%Y")+" to "+\
                boundary_date_end.strftime("%d.%m.%Y")
            charts_datas[planning_id].set_title(title)

        return charts_datas
        
Planning_by_description('report.planning_by_description')