# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA - http://www.camptocamp.com
# Author: Arnaud Wüst
#
#    This file is part of the c2c_budget module
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from c2c_reporting_tools.reports.standard_report import StandardReport
from c2c_reporting_tools.translation import _
from c2c_reporting_tools.flowables.simple_row_table import *

from c2c_planning_management.workload_tools import *

import pooler
import netsvc
import copy
import Queue
from datetime import datetime, timedelta
import time

        
        
        
        
            
class WorkloadReport(StandardReport):  
    
    
    #properties used by methods as a cache for their result
    _lines = None
    _users = None
    _lines_boundaries = None
    _working_boundaries = None
    _report_boundaries = None
    
    scale = None
    conflicts_only = None
    forced_date_start = None
    forced_date_end = None
    
    def _init(self):
        """ initialize class properties with data from the form"""
        
        
        #
        # Init cache properties
        #
        
        self._lines = None
        self._users = None
        self._lines_boundaries = None
        self._working_boundaries = None
        self._report_boundaries = None

        #
        # data from the form
        #
 
 
        self.scale = 'day'
        if 'form' in self.datas and 'scale' in self.datas['form']:
            self.scale = self.datas['form']['scale']
                    

        self.conflicts_only = False
        if 'form' in self.datas and 'conflicts_only' in self.datas['form']:
            self.conflicts_only = self.datas['form']['conflicts_only']

        self.department_ids = []
        if 'form' in self.datas and 'department' in self.datas['form']:
            self.department_ids = self.datas['form']['department'][0][2]
            
        self.forced_date_start = None
        if 'form' in self.datas and 'date_start' in self.datas['form'] \
            and self.datas['form']['date_start'] != False:
            self.forced_date_start = \
                datetime(*(
                            time.strptime(self.datas['form']['date_start'], 
                            "%Y-%m-%d")[0:6]
                            )
                        )           
            
            
        self.forced_date_end = None
        if 'form' in self.datas and 'date_end' in self.datas['form'] and self.datas['form']['date_end'] != False:
            self.forced_date_end =\
                datetime(*(
                            time.strptime(self.datas['form']['date_end'], 
                            "%Y-%m-%d")[0:6]
                            )
                        ) 
        

        
        
    def _get_lines_boundary_dates(self):   
        """ return the first day of the earliest planned task 
        to the last day of the latest planned task"""
       
        if self._lines_boundaries == None: 
            min_day = None;
            max_day = None;
            
            lines = self._get_lines()
            for l in lines:
                if min_day == None or l.date_from < min_day:
                    min_day = l.date_from
                if max_day == None or l.date_to > max_day:
                    max_day = l.date_to 
                    
            if min_day == None:
                return []
            
            min_day = datetime(*(time.strptime(min_day, "%Y-%m-%d")[0:6])) 
            max_day = datetime(*(time.strptime(max_day, "%Y-%m-%d")[0:6]))
            
            self._lines_boundaries = (min_day, max_day)
            
        return self._lines_boundaries
    
    
    
    def _get_working_boundary_dates(self):
        """ return the earliest and latest dates define in this class. 
        either the ones define in the wizard or the ones from planning
        lines' boundaries"""
        
        if self._working_boundaries == None:
        
            (lines_min, lines_max) = self._get_lines_boundary_dates()
            
            start_date = lines_min
            if self.forced_date_start != None and \
                self.forced_date_start < start_date:
                start_date = self.forced_date_start
                
            end_date = lines_max
            if self.forced_date_end != None and self.forced_date_end > end_date:
                end_date = self.forced_date_end
            
            self._working_boundaries = (start_date, end_date)
        return self._working_boundaries
    


    def _get_report_boundary_dates(self):
        """ return the first day and the last day to display in the report. 
        The dates defined in the wizard or the planning lines' boundary dates 
        if wizard's dates are not defined"""

        if self._report_boundaries == None:
        
            (lines_min, lines_max) = self._get_lines_boundary_dates()
            start_date = lines_min
            if self.forced_date_start != None:
                start_date = self.forced_date_start
                
            end_date = lines_max
            if self.forced_date_end != None:
                end_date = self.forced_date_end

            self._report_boundaries = (start_date, end_date)
        return self._report_boundaries

    
    
    def get_story(self):
        """ return the elements of the report """
        
        story = []
        #to be sure to initate cache peroperties
        self._init()
                   
        # one table per period
        (start, end) = self._get_report_boundary_dates()
        periods = self._get_periods(start, end, self.scale)
        
        for p in periods: 
            table = self.build_period_table(p)
            if table != False:
                story.append(table)
                story.append(Spacer(10,10))

        return story



    def get_user_ids(self):
        """return the list of users to display in the report"""
        
        #find users from the planning lines
        lines = self._get_lines()
        user_ids = []        
        for l in lines:
            if l.user_id.id not in user_ids :
                user_ids.append(l.user_id.id)
            
        
        #init a queue with selected departments
        department_queue = Queue.Queue()
        if self.department_ids != False:
            for d in self.department_ids:
                department_queue.put_nowait(d)
        
        
        #loop through selected departments and their children               
        while not department_queue.empty():
            dep_id = department_queue.get_nowait()
            
            department = self.pool.get('hr.department').browse(
                                                                self.cr, 
                                                                self.uid, 
                                                                dep_id, 
                                                                self.context
                                                            )
            
            #save its users
            for m in department.member_ids:
                if m.id not in user_ids :
                    user_ids.append(m.id)
            
            #add its children to the queue
            for c in department.child_ids:
                department_queue.put_nowait(c.id)                        
                
        sorted_user_ids = self.pool.get('res.users').search(
                                                        self.cr, 
                                                        self.uid, 
                                                        [
                                                            ("id", "in", user_ids)
                                                        ], 
                                                        order="name ASC"
                                                    )    
        return sorted_user_ids
        
        
    def _get_users(self):    
        """return the list of users' objects to display in the report """
        
        if self._users == None:
            user_ids = self.get_user_ids()
            users = self.pool.get('res.users').browse(
                                                        self.cr, 
                                                        self.uid, 
                                                        user_ids, 
                                                        self.context
                                                    )

            self._users = []
            for u in users:
                (boundary_start, boundary_end) = self._get_working_boundary_dates() 
                #return the limits of this report. NO infomration will be needed 
                #beyond those date
                tmp_user= User(u, boundary_start, boundary_end) 
                #give dates to user in order to allow it to optimize computing
                #search for lines related to this user
                for l in self._get_lines():
                    if l.user_id.id == tmp_user.obj.id: 
                        tmp_user.add_line(l)
                self._users.append(tmp_user)
                
        return self._users
    
    
    def _get_lines(self):
        """ return the planning lines """   
        
        if self._lines == None:    
            #find users from the planning lines
            self._lines = self.pool.get('report_account_analytic.planning.line').browse(self.cr, self.uid, self.ids)
    
        return self._lines
    
    


    def _get_periods(self, date_start, date_end, scale):
        """ return the list of periods to display """
        
        if scale == 'month':
            return self._get_periods_by_month(date_start, date_end)
        elif scale == 'week':
            return self._get_periods_by_week(date_start, date_end)
        else:
            return self._get_periods_by_day(date_start, date_end)
        
        
    def _get_periods_by_day(self, date_start, date_end):
        """return periods definition by day """
        
        # the list of working days
        periods = []
        
        current_date = date_start
        while current_date <= date_end:
            day = Day(current_date)
            if current_date.weekday() in [0,1,2,3,4]:
                day.work_day = True
            else:
                day.work_day = False

            periods.append(day)
            current_date = current_date + timedelta(days=1)
        return periods

        
    def _get_periods_by_week(self, date_start, date_end):
        """return periods definition by week """
    
        periods = []
        previous_day = None

        #find the first day of the week of date_from
        date_from = c2c_helper.week_first_day(date_start)
        
        #find the last day of the week of date_to
        date_to = c2c_helper.week_last_day(date_end)
        
        days = self._get_periods_by_day(date_from, date_to)
        
        current_period_start = date_start
        for current_day in days:
            #is it a monday? (and it's not the date_from given in parameters)
            if current_day.date_from.weekday() == 0 and previous_day != None:
                #let's end the current week and start a new one!
                week = Week(current_period_start, previous_day.date_to)    
                current_period_start = current_day.date_from        
                periods.append(week)

            previous_day = current_day

        #at the end of the loop, let's add the last time_unit 
        #(this one finish at date_to instead of the friday)
        week = Week(current_period_start, date_end)
        periods.append(week)
        return periods
       
       
    def _get_periods_by_month(self, date_start, date_end):
        """return periods definition by months """
        
        previous_day = None
        periods = []


        #find the first day of the week of date_from
        date_from = c2c_helper.month_first_day(date_start)
        
        #find the last day of the week of date_to
        date_to = c2c_helper.month_last_day(date_end)

        days = self._get_periods_by_day(date_start, date_end)
        
        current_period_start = date_start
        for current_day in days:
            #is it the first day of the month? (and it's not the date_from given in parameters)
            if current_day.date_from.strftime("%d") == "01" and previous_day != None:
                month = Month(current_period_start, previous_day.date_to)
                current_period_start = current_day.date_from
                periods.append(month)
                
            previous_day = current_day
        
        month = Month(current_period_start, date_end)
        periods.append(month)
        return periods
       
    
    
    def build_period_table(self, period):
        #build the table. First define columns
        
        table_empty = True
        
        builder = SimpleRowsTableBuilder(period.name)
        builder.add_text_column(_('User'), "15%", 2)
        builder.add_num_column(_('Planned Time [h]'), "10%", 2)
        builder.add_num_column(_('Available Time [h]'), "10%", 2)
        builder.add_num_column(_('Remaining Hours [h]'), "10%", 2)
    
        #a nested table allows for one main row, to have multiple aligned
        #rows in specifics columns
        nested_table = SimpleRowsTableBuilder()
        nested_table.add_text_column(_('Projects'), 'auto', 2)
        nested_table.add_text_column(_('Activities'), 'auto', 2)        
        builder.add_nested_columns(nested_table)

        #user are listed below each period
        users = self._get_users()
        for user in users:
            workload = user.get_workload(self.cr, self.uid, period)
            working_time = user.get_working_time(self.cr, self.uid, period)
            
            if (not self.conflicts_only or workload > working_time):
                
                builder.add_text_cell(user.obj.name)
                
                builder.add_num_cell(workload)
                
                builder.add_num_cell(working_time)
                
                builder.add_num_cell(working_time-workload)
                
                #copy the nested table used to define columns
                tmp_table = copy.deepcopy(nested_table)
                #add list of values for this row
                for proj in user.get_projects(self.cr, self.uid, period):
                    (project, activity) = proj
                    tmp_table.add_text_cell(project)
                    tmp_table.add_text_cell(activity)
                builder.add_nested_cells(tmp_table)
                
                table_empty = False
                
        
        if not table_empty:
            return builder.get_table()
    
        return False           
           
WorkloadReport('report.workload', 
                "Workload", 
                "report_account_analytic.planning.line", 
                StandardReport.A4_PORTRAIT
            )        